<?php
/**
 * Plugin Name: OC Analyzer
 * Description: Log and analyze OC queries
 * Version: 0.0.1
 * Author: Infomaker Scandinavia AB
 * Author URI: http://infomaker.se
 *
 * Creator: Andreas Kihlberg - andreas.kihlberg@infomaker.se
 */

require dirname(__FILE__)."/ocanalyzer/action.php";


// Define plugin database version and a request id
global $ocanalyzer_db_version, $ocanalyzer_request_id;
$ocanalyzer_db_version = '1.0';
$ocanalyzer_request_id = uniqid();


// Install and uninstall database
register_activation_hook( __FILE__, 'ocanalyzer_install' );
register_deactivation_hook( __FILE__, 'ocanalyzer_uninstall_data' );


// Add OC Analyzer to OC Menu page
add_action('admin_menu', function() {
    add_submenu_page( 'oc_connection', 'OC Analyzer', __('OC Analyzer', 'every'), 'manage_options', 'ocanalyzer_page', 'ocanalyzer_page'  );
});

function get_sort_order() {
    $by = "id";
    $sort = "DESC";

    if(isset($_GET['order'])) {
        $by = $_GET['order'];
        $sort = $_GET['sort'];
    }

    return [
        'by'    => $by,
        'sort'  => $sort
    ];
}

function ocanalyzer_page() {

    global $wpdb;

    if(isset($_POST['truncate'])) {
        $wpdb->query("TRUNCATE TABLE ".$wpdb->prefix."ocanalyzer");
    }

    $currentpage = isset($_GET['p']) ? $_GET['p'] : 0;
    $itemsPerPage = isset($_GET['rpp']) ? $_GET['rpp'] : 100;
    $start = $currentpage*$itemsPerPage;


    $lyzer = Action::getInstance();

    // Extract sort parameter from get_sort_order()
    extract(get_sort_order());

    $q = $wpdb->get_results("SELECT count(id) as items FROM ".$wpdb->prefix."ocanalyzer  ");
    $totalItems = $q[0]->items;
    $rows = $wpdb->get_results("SELECT * FROM ".$wpdb->prefix."ocanalyzer ORDER BY $by $sort LIMIT $start, $itemsPerPage");

    if(isset($_GET['group'])) {

        $field = $_GET['group'];
        $q = $wpdb->get_results("SELECT count(argument) as items FROM ".$wpdb->prefix."ocanalyzer group by ".$field."  ");
        $totalItems = $q[0]->items;
        $rows = $wpdb->get_results("select *, count(argument) as number from ".$wpdb->prefix."ocanalyzer group by ".$field." ORDER BY number DESC LIMIT $start, $itemsPerPage");
    }

    if(isset($_GET['single'])) {
        $id = $_GET['single'];
        $row = $wpdb->get_results("select * from ".$wpdb->prefix."ocanalyzer WHERE id = $id LIMIT $start, $itemsPerPage");
        renderItem($row);
    }

    if(isset($_GET['ocanalyzer_id'])) {
        $id = $_GET['ocanalyzer_id'];
        $analyzerRows = $wpdb->get_results("select * from ".$wpdb->prefix."ocanalyzer WHERE ocanalyzer_id = '$id'");
        renderHeader("Request " . $id . " genereated following requests");
        renderItems($analyzerRows);
    }

    $url = $_SERVER['REQUEST_URI'];
    if(strpos($url, '&p=') === false) {
        $url .= '&p=0';
    }

    $old =  'p='.$currentpage;
    $pageString = 'p='.($currentpage+1);
    $pageStringPrev = 'p='.($currentpage-1);

    $next = str_replace($old, $pageString, $url);
    $prev = str_replace($old, $pageStringPrev, $url);

    echo "<br>";
    echo getStyle();
    ?>

    <div class='wrap'>
        <h2 class="oc_Settings_header">Senaste</h2>

        <strong>Number of rows <?php echo $totalItems;?></strong><br>

        <form method="post" style="float: right;" name="truncate_table" onsubmit="javascript: return confirm('Delete?');">
            <input type="submit" name="truncate" value="Truncate"/>
        </form>

       <?php echo get_pagination_links($prev, $next); ?>

        <table border="0" class="ocanalyzer_data" cellspacing="0" cellpadding="0">
            <tr>
                <td>Time</td>
                <td>Request time  <a style="float:right" href="admin.php?page=ocanalyzer_page&group=start_request">Group</a></td>
                <td>Type  <a style="float:right" href="admin.php?page=ocanalyzer_page&group=name">Group</a></td>
                <td>Called function  <a style="float:right" href="admin.php?page=ocanalyzer_page&group=function">Group</a></td>
                <td>OC URL requested <a style="float:right" href="admin.php?page=ocanalyzer_page&group=argument">Group</a></td>
                <td>URI Requested<a style="float:right" href="admin.php?page=ocanalyzer_page&group=request_uri">Group</a></td>
                <td> Loadtime
                    <a style="float:right; display: inline-block; padding:0 3px;" href="admin.php?page=ocanalyzer_page&order=loadtime&sort=desc">&darr;</a>
                    <a style="float:right; display: inline-block; padding:0 3px;" href="admin.php?page=ocanalyzer_page&order=loadtime&sort=asc">&uarr;</a>
                </td>
                <td>Request ID<a style="float:right" href="admin.php?page=ocanalyzer_page&group=ocanalyzer_id">Group</a></td>
                <td>Number</td>
            </tr>

            <?php
                $colorArray = [];
                $i=0; foreach($rows as $row) :
                    $color = getRandomColor($colorArray, $row->ocanalyzer_id);
                ?>

                <tr class="<?php if($i % 2 == 0) {echo 'alt'; }?> <?php if($row->id == $_GET['single']) {echo 'active'; }?> ">
                    <td style='border-left:10px solid <?php echo $color;?>'><a href="admin.php?page=ocanalyzer_page&single=<?php echo $row->id;?>"><?php echo date('H:i:s', strtotime($row->time)) ;?></a></td>
                    <td><a href="admin.php?page=ocanalyzer_page&request=<?php echo $row->start_request;?>"><?php echo $row->start_request ;?></a></td>
                    <td><?php echo $row->type ;?></td>
                    <td><?php echo $row->function ;?></td>
                    <td>
                    <a href="<?php echo $row->argument;?>"><?php echo $lyzer->shortenUrl($row->argument) ;?></a>
                    </td>
                    <td>
                    <a href="<?php echo $row->request_uri;?>"><?php echo $lyzer->shortenUrl($row->request_uri) ;?></a>
                    </td>
                    <td><?php echo round($row->loadtime,3) ;?> s</td>
                    <td>
                        <a href="admin.php?page=ocanalyzer_page&ocanalyzer_id=<?php echo $row->ocanalyzer_id;?>"><?php echo $row->ocanalyzer_id ;?></a>
                    </td>
                    <td>
                        <?php
                        if(isset($row->number)) {
                            echo "<strong>".$row->number."</strong>";
                        }
                        ?>
                    </td >
                </tr>

            <?php $i++; endforeach; ?>

        </table>

         <?php echo get_pagination_links($prev, $next); ?>

    <div>
<?php
}


function renderItem($row) {
    $row = $row[0];
    $data = unserialize($row->data);
    $functions = $data['functions'];
    $item = $data['data'][0];

    ?>

    <div class="item">
        <div class="half">
            <h3><?php echo $row->name;?></h3>
            <p>Url called: <a href="<?php echo  $row->argument;?>"><?php echo $row->argument; ?></a></p>
            <p>File used: <?php echo $row->file; ?> @ line <?php echo $item['line'];?></p>
        </div>
        <div class="half">
            <h3>Functions called before</h3>
            <table border="0" class="ocanalyzer_data" cellspacing="0" cellpadding="0">

                <?php $i=0;  foreach($functions as $f) :?>
                <tr class='<?php if($i % 2 == 0) {echo 'alt'; }?>'>
                    <td><?php echo $f; ?></td>
                </tr>
                    <?php $i++; if($i == 10) { break; } endforeach; ?>
            </table>
        </div>
        <div class="clearfix"></div>
    </div>
<?php

}


function renderItems($rows) {

    $lyzer = Action::getInstance();
    ?>

    <table border="0" class="ocanalyzer_data" cellspacing="0" cellpadding="0">

        <?php
        $i=0;
        $loadtime = 0;
        foreach ($rows as $row):

            ?>
            <tr class='<?php if($i % 2 == 0) {echo 'alt'; }?>'>
                <td><a href="admin.php?page=ocanalyzer_page&single=<?php echo $row->id;?>"><?php echo $row->time ;?></a></td>
                <td><?php echo $row->start_request; ?></td>
                <td><?php echo $row->type; ?></td>
                <td><?php echo $row->file; ?></td>
                <td><?php echo $row->function; ?></td>
                <td>
                <a href="<?php echo $row->argument;?>"><?php echo $lyzer->shortenUrl($row->argument) ;?></a>
                </td>
                <td>
                <a href="<?php echo $row->request_uri;?>"><?php echo $lyzer->shortenUrl($row->request_uri) ;?></a>
                </td>
                <td><?php $loadtime += $row->loadtime; echo round($row->loadtime,4); ?> s</td>
            </tr>
        <?php  $i++; endforeach;?>

        <tr>
            <td colspan="7"></td>
            <td><strong><?php echo round($loadtime,4);?> s</strong></td>
        </tr>
    </table>

    <?php
}


function get_pagination_links($prev, $next)
{
    ?>
    <div class="tablenav-pages pagination">
        <span class="pagination-links">
            <a href="<?php echo $prev;?>" class="prev-page">Previous</a>
            <a href="<?php echo $next;?>" class="next-page">Next</a>
        </span>
    </div>

<?php
}


function renderHeader($string) {
    echo "<h2>".$string."</h2>";
}

function getStyle() {
    return " <style>
        .ocanalyzer_data {
            border:1px solid #ccc;
            width: 100%;
        }
        .ocanalyzer_data tr td {
            border-bottom:1px solid #ccc;
            border-right:1px solid #ccc;
            padding:5px;
        }
        .ocanalyzer_data .alt {
            background-color: #fff;;
        }
         .ocanalyzer_data .active {
            background-color: #272727;
            color: #fff;
        }
        .ocanalyzer_data .active  a{
            color: #fff;
        }
        .half {
            float:left;
            width: 50%;
        }
        .item {
            border-bottom: 1px solid #ccc;
            padding-bottom: 40px;;
        }
        .pagination {
            display: block;
            margin: 10px 0;
        }
    </style>";
}
function add_data_callback($arg, $type) {

    $lyzer = Action::getInstance();
    $lyzer->add($arg, $type);
}

function get_data_callback() {

    $lyzer = Action::getInstance();
    $lyzer->render();
}

function get_data_database_callback($request) {

    $lyzer = Action::getInstance();
    $lyzer->renderDatabase($request);
}


function getRandomColor(&$array, $key) {

    if(array_key_exists($key, $array)) {
        return $array[$key];
    } else {
        $array[$key] = 'rgb('.rand(0,255).', '.rand(0,255).', '.rand(0,255).')';
        return $array[$key];
    }

}




add_action('analyzer_add_data', 'add_data_callback', 10, 2);
add_action('analyze_get_data', 'get_data_callback');
add_action('analyze_get_data_database', 'get_data_database_callback', 10, 1);


/**
 * Installation of database
 */

function ocanalyzer_install() {
    global $wpdb;
    global $ocanalyzer_db_version;

    $table_name = $wpdb->prefix . 'ocanalyzer';
    $charset_collate = '';

    if ( ! empty( $wpdb->charset ) ) {
        $charset_collate = "DEFAULT CHARACTER SET {$wpdb->charset}";
    }

    if ( ! empty( $wpdb->collate ) ) {
        $charset_collate .= " COLLATE {$wpdb->collate}";
    }

    $sql = "CREATE TABLE $table_name (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
		name VARCHAR (255) NOT NULL,
		function VARCHAR (255) NOT NULL,
		file VARCHAR (255) NOT NULL,
		argument VARCHAR (255) NOT NULL,
		data text NOT NULL,
		type VARCHAR(20) NOT NULL,
		ocanalyzer_id VARCHAR (40) NOT NULL,
		start_request VARCHAR (40) NOT NULL,
		loadtime VARCHAR (40) NOT NULL,
		request_uri VARCHAR (255) NOT NULL,

		UNIQUE KEY id (id)
	) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta( $sql );

    add_option( 'ocanalyzer_db_version', $ocanalyzer_db_version );
}

function ocanalyzer_uninstall_data() {

    global $wpdb;
    $table = $wpdb->prefix."ocanalyzer";
    $wpdb->query("DROP TABLE IF EXISTS $table");
}
